-- Lua Script1
-- Author: zoggop
-- DateCreated: 3/28/13 11:19:04 PM
--------------------------------------------------------------
include("math")
include("MapGenerator")
include("FractalWorld")
include("FeatureGenerator")
include("TerrainGenerator")

function GetMapScriptInfo()
	return {
		Name = "Central Valley",
		Description = "a wide green valley in the middle, surrounded by mountains and snow",
		IconIndex = 5,
	}
end

----------------------------------------------------------------------------------

local function getDistance(x1, y1, x2, y2)
	local xdist = math.abs(x1 - x2)
	local ydist = math.abs(y1 - y2)
	local distance = math.floor(math.sqrt((xdist*xdist) + (ydist*ydist)))
	return distance
end

local heightmap = {}
local oceanLevel = 0.35
local hillsLevel = 0.6
local mountainLevel = 0.9
local valleyRadius = 10
local isValley = {}
local thePercent = {}

function GeneratePlotTypes()
	print("Setting Plot Types (Central Valley) ...");

	local iW, iH = Map.GetGridSize()
	local centerX = iW / 2
	local centerY = iH / 2
	local maxDistance = 0
	if iW < iH then
		local xdist = iW / 2
		local ydist = iW / 2
		maxDistance = math.floor(math.sqrt((xdist*xdist) + (ydist*ydist)))
	else
		local xdist = iH / 2
		local ydist = iH / 2
		maxDistance = math.floor(math.sqrt((xdist*xdist) + (ydist*ydist)))
	end
	local plotTypes = {}
	for y = 0, iH - 1 do
            for x = 0, iW - 1 do
				local plotIndex = y * iW + x + 1
				local distance = getDistance(x, y, centerX, centerY)
				local percent = 100 * (distance / maxDistance)
				thePercent[plotIndex] = math.floor(percent)
				local randadd = (math.random() - 0.5) * 0.1
				local height =  0.36
				isValley[plotIndex] = true
				if percent > valleyRadius then
					height = math.exp(-((percent-valleyRadius) / 40)) + randadd
					isValley[plotIndex] = false
				end
				heightmap[plotIndex] = height
				if height < oceanLevel then
					plotTypes[plotIndex] = PlotTypes.PLOT_OCEAN
				elseif height < hillsLevel then
					plotTypes[plotIndex] = PlotTypes.PLOT_LAND
				elseif height < mountainLevel then
					plotTypes[plotIndex] = PlotTypes.PLOT_HILLS
				else
					plotTypes[plotIndex] = PlotTypes.PLOT_MOUNTAIN
				end
			end
	end
	
	SetPlotTypes(plotTypes);
	GenerateCoasts();

end

----------------------------------------------------------------------------------

function GenerateTerrain()
	print("Generating Terrain (Central Valley) ...");
	
	local terrainDesert	= GameInfoTypes["TERRAIN_DESERT"];
	local terrainPlains	= GameInfoTypes["TERRAIN_PLAINS"];
	local terrainSnow	= GameInfoTypes["TERRAIN_SNOW"];
	local terrainTundra	= GameInfoTypes["TERRAIN_TUNDRA"];
	local terrainGrass	= GameInfoTypes["TERRAIN_GRASS"];
	
	local freezingTemp = 32
	
	local iW, iH = Map.GetGridSize()
	local terrainTypes = {}
	for plotIndex,percent in ipairs(thePercent) do
		if isValley[plotIndex] then
			terrainTypes[plotIndex] = terrainGrass
		else
			local randadd = math.random(0,10) - 5
			local fuzz = percent + randadd
			if fuzz < valleyRadius then
				terrainTypes[plotIndex] = terrainGrass
			elseif fuzz < valleyRadius + 20 then
				terrainTypes[plotIndex] = terrainSnow
			elseif fuzz < valleyRadius + 25 then
				terrainTypes[plotIndex] = terrainTundra
			elseif fuzz < valleyRadius + 35 then
				terrainTypes[plotIndex] = terrainDesert
			elseif fuzz < valleyRadius + 45 then
				terrainTypes[plotIndex] = terrainPlains
			else
				terrainTypes[plotIndex] = terrainGrass
			end
		end
	end
		
	SetTerrainTypes(terrainTypes);
end

----------------------------------------------------------------------------------

function FeatureGenerator:AddIceAtPlot(plot, iX, iY, lat)
	if(plot:CanHaveFeature(self.featureIce)) then

	end
end

function AddFeatures()
	print("Adding Features (Lua Fractal) ...");
	
	local featuregen = FeatureGenerator.Create();
	featuregen:AddFeatures(false);
end

----------------------------------------------------------------------------------